##########################################################
# 
# A trivial file transfer layer on top of DP.
#	mperham 3/97
# 
# Assumes a server has been started with "fserver"
# and a client connected to that server via "fconnect"
#
# The user interface:
#
#    fconnect $host {$port} - open an RPC connection to host:port
#			    and a data connection to host:port-1
#			    port is optional
#    fget $filename - copy remote file, $filename, to local machine.
#    fput $filename - copy local file, $filename, to remote machine.
#    fdel $filename - delete a remote file named $filename
#    fdir - remote directory listing
#    fpwd - print current remote directory
#    fcd - change remote directory
#
# Problems:
#
#    Minimal error handling at best.  Need to catch more.
#

package require dp 4.0

#//////////////////////////////////////

proc oops {errMsg} {
    puts stdout "Remote server returned error:"
    puts stdout "$errMsg"
}

#//////////////////////////////////////

proc fget {filename} {
    global f_chan, dataPort

# Make data channel

    if {[catch {dp_RDO $f_chan eval {set dataServ \[dp_connect -server 1\
	    -myport $dataPort\]; set dataSC \[dp_accept \$dataServ\]}} msg]} {
	puts stdout "Error opening dataServer socket: $msg"
	close $f_chan
	return
    }
    if {[catch {set dataClient [dp_connect tcp -host $host -port $dataPort]}
	    msg]} {
    	puts stdout "Error connecting to dataServer socket: $msg"
    	close $f_chan
    	return
    }

# Tell server to send file

    dp_RDO $f_chan -onerror oops "set daFile \[open $filename r\];\
	     dp_send \$dataSC \[dp_recv \$daFile\];\
	     close \$dataSC;\
	     close \$daFile"

    set newfile [open $filename {CREAT WRONLY}]
    puts stdout "Getting $filename from remote host."
    while {![eof $dataClient]} {
	dp_send $newfile [dp_recv $dataClient]
	puts stdout "#" nonewline
    }
    close $newfile
    close $dataClient
}

#//////////////////////////////////////

proc fput {filename} {
    global f_chan, dataPort

# Make data channel

    if {[catch {dp_RDO $f_chan eval {set dataServ \[dp_connect -server 1\
	    -myport $dataPort\]; set dataSC \[dp_accept \$dataServ\]}} msg]} {
	puts stdout "Error opening dataServer socket: $msg"
	close $f_chan
	return
    }
    if {[catch {set dataClient [dp_connect tcp -host $host -port $dataPort]}
	    msg]} {
    	puts stdout "Error connecting to dataServer socket: $msg"
    	close $f_chan
    	return
    }

# Tell server to recv file

    dp_RDO $f_chan "set daFile \[open $filename w\];\
	    dp_send \$daFile \[dp_recv \$dataSC\];\
	    close \$dataSC;\
	    close \$daFile"

    set newfile [open $filename r]
    puts stdout "Putting $filename to remote host."
    while {![eof $newfile]} {
	dp_send $dataClient [dp_recv $newfile]
        puts stdout "#" nonewline
    }
    puts stdout ""
    close $dataClient
    close $newfile
}

#//////////////////////////////////////

proc fpwd {} {
    global f_chan

    dp_RPC $f_chan "if {$tcl_version(platform) == "windows"}\
    			{ [exec cmd.exe /c cd] } else { pwd }"
}

#//////////////////////////////////////

proc fdir {} {
    global f_chan

# I think this will be broken for filenames with spaces.
# The foreach will think they are different files.

    foreach a [dp_RPC $f_chan glob \*] {
	puts $a
    }
}

#//////////////////////////////////////

proc fdel {filename} {
    global f_chan

    dp_RPC $f_chan file delete $filename
}

#//////////////////////////////////////

proc fcd {dir} {
    global f_chan

    dp_RDO $f_chan -onerror oops cd $dir
}

#//////////////////////////////////////

proc fquit {} {
    global f_chan

    close $f_chan
}

#//////////////////////////////////////
# We need to open one channel on each side; the
# the data channel - $dataClient here, $dataSC on the server -
# will be created each time we send or recv a file
# since we can't send normal data over an RPC channel.

proc fconnect {host {port 19743}} {
    global f_chan, dataPort

    set f_chan [dp_MakeRPCClient $host $port]
    puts stdout "Connected to $host..."
    set dataPort [incr $port -1]
    
    puts stdout "--------- Directory Listing ----------"
    fdir
    puts stdout "--------- Remote Directory -----------"
    fpwd
}
